#include "control_masina.h"

/*********************************USART FUNCTIONS *****************************/


void usart_init(void)
{
    /* seteaza baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 103;

    /* porneste transmitatorul */
    UCSR0B |= (1 << RXEN0);

    /* frame de 8biti de date */
    UCSR0C |= (1 << UCSZ00 | 1 << UCSZ01);

    /* 1 bit de stop */
    UCSR0C &= ~(1 << USBS0);

    /*fara paritate*/
    UCSR0C &= ~(1 << UPM00 | 1 << UPM01);
}

void usart_transmit(char data)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
}


char usart_receive(void)
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

void usart_print(const char *data)
{
    while(*data != '\0')
        usart_transmit(*data++);
}

/********************************************************************************************/
void set_speed(char value){

	//mergem inainte
	value &= 0x0F;

	if ((value ^ FORWARD0) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_FB);
		OCR1A = 220;
	}
	if ( (value ^ FORWARD1) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_FB);
		OCR1A = 90;
		
	}
	if (( value ^ FORWARD2) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_FB);
		OCR1A = 60;
		
	}
	if ( (value ^ FORWARD3) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_FB);
		OCR1A = 40;
	}
	if ( (value ^ BACKWARD0) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_FB);
		OCR1A = 20;
		
	}
	if ( (value ^ BACKWARD1) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_FB);
		OCR1A = 150;
		
	}
	if ( (value ^ BACKWARD2) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_FB);
		OCR1A = 180;
		
	}
	if ( (value ^ BACKWARD3) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_FB);
		OCR1A = 230;
		
	}
}

void set_direction(char value)
{

	//mergem inainte
	value &= 0xF0;

	if ((value ^ LEFT0) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_LR);
		OCR1B = 220;
	}
	if ( (value ^ LEFT1) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_LR);
		OCR1B = 90;
		
	}
	if (( value ^ LEFT2) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_LR);
		OCR1B = 60;
		
	}
	if ( (value ^ LEFT3) == 0x00)
	{
		PORTB |= (1 << MOTOR_DIRECTIE_LR);
		OCR1B = 40;
	}
	if ( (value ^ RIGHT0) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_LR);
		OCR1B = 20;
		
	}
	if ( (value ^ RIGHT1) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_LR);
		OCR1B = 150;
		
	}
	if ( (value ^ RIGHT2) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_LR);
		OCR1B = 180;
		
	}
	if ( (value ^ RIGHT3) == 0x00)
	{
		PORTB &= ~( 1 << MOTOR_DIRECTIE_LR);
		OCR1B = 230;
		
	}
}

void turn_lights_on(void)
{
	PORTA |= (1 << PA0);
	PORTA |= (1 << PA1);

}
void turn_lights_off(void)
{
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
}


void control_car(void)
{

	char state;


	/* Setez farurile */
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);


	/* Setez  motoarele cu PWM */
	DDRD |= (1 << MOTOR_PWM_FB);
	DDRD |= (1 << MOTOR_PWM_LR);

	/*Setez motoarele de directie */
	DDRB |= (1 << MOTOR_DIRECTIE_LR);
	DDRB |= (1 << MOTOR_DIRECTIE_FB);


	/* Sting LED-ul. */
	PORTB &= ~(1 << MOTOR_DIRECTIE_FB);
	PORTB &= ~(1 << MOTOR_DIRECTIE_LR);

	/*Setare PWM pentru motoare */
	
	//Conform tabelului 16-5 din datasheet, pentru modul Fast PWM 8-bit, biții WGMn0 si WGMn2 au valoarea 1
	TCCR1A |= (1 << WGM10); 
	//TCCR1A conține doar biții WGM10 si WGM11, WGM12 și WGM13 se găsesc in TCCR1B
	TCCR1B |= (1 << WGM12);

	//Conform tabelului 16-3 din datasheet, pentru modul non-inverting COM1A1 = 1 și COM1A0 = 0
	TCCR1A |= (1 << COM1A1);
	TCCR1A |= (1 << COM1B1);

	//Conform tabelului 16-6 din datasheet, pentru Prescaler de 1024 scriem 1 pe CS12 si CS10
	TCCR1B |= (1 << CS12) | (1 << CS10);
	//Pragul la care se schimbă semnalul (Fig. 4); fill-rate 0.5: jumătate din timp 1, jumătate din timp 0
	//Deoarece în acest mod TOP este 0xFF, OCR1A va fi 50 * 255 / 100
	OCR1A = 1;
	OCR1B = 1;

	while (1) {


		state = usart_receive();
		if ((state ^ LIGHTS_ON) == 0x00)
		{
			PORTD |= (1 << PD7);
			turn_lights_on();
		}
		else if ((state ^ LIGHTS_OFF) == 0x00)
		{
			PORTD &= ~(1 << PD7);
			turn_lights_off();

		} else {
			set_speed(state);
			set_direction(state);
		} 
	}
}


int main(void) {

	usart_init();
	control_car();
	return 0;
}






